@@ -747,6 +747,12 @@ struct MHD_Connection
    */
   size_t write_buffer_append_offset;
 
+  /**
+   * Number of bytes we had in the HTTP header, set once we
+   * pass #MHD_CONNECTION_HEADERS_RECEIVED.
+   */
+  size_t header_size;
+
   /**
    * How many more bytes of the body do we expect
    * to read? #MHD_SIZE_UNKNOWN for unknown.
@@ -1540,6 +1546,11 @@ struct MHD_Daemon
    */
   uint16_t port;
 
+  /**
+   * Be neutral (zero), strict (1) or permissive (-1) to client.
+   */
+  int strict_for_client;
+
 #ifdef HTTPS_SUPPORT
 #ifdef UPGRADE_SUPPORT
   /**
