@@ -50,8 +50,8 @@ test_concurent_daemon_pair (void *cls,
   struct MHD_Daemon *d1;
   struct MHD_Daemon *d2;
 
-  d1 = MHD_start_daemon (MHD_USE_THREAD_PER_CONNECTION | MHD_USE_TLS |
-                         MHD_USE_DEBUG, DEAMON_TEST_PORT,
+  d1 = MHD_start_daemon (MHD_USE_THREAD_PER_CONNECTION | MHD_USE_INTERNAL_POLLING_THREAD | MHD_USE_TLS |
+                         MHD_USE_ERROR_LOG, DEAMON_TEST_PORT,
                          NULL, NULL, &http_ahc, NULL,
                          MHD_OPTION_HTTPS_MEM_KEY, srv_key_pem,
                          MHD_OPTION_HTTPS_MEM_CERT, srv_self_signed_cert_pem,
@@ -63,8 +63,8 @@ test_concurent_daemon_pair (void *cls,
       return -1;
     }
 
-  d2 = MHD_start_daemon (MHD_USE_THREAD_PER_CONNECTION | MHD_USE_TLS |
-                         MHD_USE_DEBUG, DEAMON_TEST_PORT + 1,
+  d2 = MHD_start_daemon (MHD_USE_THREAD_PER_CONNECTION | MHD_USE_INTERNAL_POLLING_THREAD | MHD_USE_TLS |
+                         MHD_USE_ERROR_LOG, DEAMON_TEST_PORT + 1,
                          NULL, NULL, &http_ahc, NULL,
                          MHD_OPTION_HTTPS_MEM_KEY, srv_key_pem,
                          MHD_OPTION_HTTPS_MEM_CERT, srv_self_signed_cert_pem,
@@ -96,6 +96,7 @@ main (int argc, char *const *argv)
 {
   unsigned int errorCount = 0;
   FILE *cert;
+  const char *aes256_sha = "AES256-SHA";
 
   gcry_control (GCRYCTL_ENABLE_QUICK_RANDOM, 0);
 #ifdef GCRYCTL_INITIALIZATION_FINISHED
@@ -105,20 +106,26 @@ main (int argc, char *const *argv)
     {
       fprintf (stderr, "Error (code: %u). l:%d f:%s\n", errorCount, __LINE__,
                __FUNCTION__);
-      return -1;
+      return 99;
+    }
+  if (NULL == curl_version_info (CURLVERSION_NOW)->ssl_version)
+    {
+      fprintf (stderr, "Curl does not support SSL.  Cannot run the test.\n");
+      curl_global_cleanup ();
+      return 77;
     }
   if ((cert = setup_ca_cert ()) == NULL)
     {
       fprintf (stderr, MHD_E_TEST_FILE_CREAT);
-      return -1;
+      curl_global_cleanup ();
+      return 99;
     }
 
-  const char *aes256_sha = "AES256-SHA";
   if (curl_uses_nss_ssl() == 0)
     {
       aes256_sha = "rsa_aes_256_sha";
     }
-  
+
   errorCount +=
     test_concurent_daemon_pair (NULL, aes256_sha, CURL_SSLVERSION_TLSv1);
 
@@ -130,5 +137,5 @@ main (int argc, char *const *argv)
     fprintf (stderr,
 	     "Failed to remove `%s'\n",
 	     ca_cert_file_name);
-  return errorCount != 0;
+  return errorCount != 0 ? 1 : 0;
 }
